import { expect, test, describe, it } from 'bun:test';
import type { GraphQueryResultMetaFromBackend, InsightModel } from 'ts-common';

import { processAlarmStats } from '../statCheck';

const baseInsight: Omit<InsightModel, 'conditionsCheck'> = {
  id: 1,
  name: 'Test Insight',
  description: 'Base insight for testing',
  recipients: ['test@example.com'],
  frequency: 'daily',
  template: 'default',
  saveStateId: 'save-state-1',
  type: 'alert',
  alarmMode: 'conditional',
  createdAt: new Date().toISOString(),
  updatedAt: new Date().toISOString(),
  previousResultHash: 'abc123',
  lastProcessedAt: new Date().toISOString(),
  status: true,
};

describe('QueryprocessAlarmStats', () => {
  it('should return true when condition is met', () => {
    const alarmStat: InsightModel = {
      ...baseInsight,
      conditionsCheck: [
        {
          nodeLabel: 'TestNode',
          statistic: 'count',
          operator: '>',
          value: 5,
        },
      ],
    };

    const resultQuery: GraphQueryResultMetaFromBackend = {
      nodes: [
        {
          _id: 'node1',
          label: 'TestNode',
          attributes: {
            name: 'Test Node 1',
            value: 123,
          },
        },
        {
          _id: 'node2',
          label: 'TestNode',
          attributes: {
            name: 'Test Node 2',
            value: 456,
          },
        },
      ],
      edges: [
        {
          _id: 'edge1',
          label: 'CONNECTS',
          from: 'node1',
          to: 'node2',
          attributes: {
            weight: 1,
            timestamp: '2025-03-06',
          },
        },
      ],
      metaData: {
        topological: {
          density: 0.5,
          self_loops: 0,
        },
        nodes: {
          count: 2,
          labels: ['TestNode'],
          types: {
            TestNode: {
              count: 2,
              avgDegreeIn: 0.5,
              avgDegreeOut: 0.5,
              attributes: {
                name: {
                  attributeType: 'string',
                  statistics: {
                    uniqueItems: 2,
                    values: ['Test Node 1', 'Test Node 2'],
                    mode: 'Test Node 1',
                    count: 2,
                  },
                },
                value: {
                  attributeType: 'number',
                  statistics: {
                    min: 123,
                    max: 456,
                    average: 289.5,
                    count: 2,
                  },
                },
              },
            },
          },
        },
        edges: {
          count: 1,
          labels: ['CONNECTS'],
          types: {
            CONNECTS: {
              count: 1,
              attributes: {
                weight: {
                  attributeType: 'number',
                  statistics: {
                    min: 1,
                    max: 1,
                    average: 1,
                    count: 1,
                  },
                },
                timestamp: {
                  attributeType: 'datetime',
                  statistics: {
                    min: 1743782400,
                    max: 1743782400,
                    range: 0,
                  },
                },
              },
            },
          },
        },
      },
      nodeCounts: {
        TestNode: 2,
        updatedAt: 122331,
      },
    };

    expect(processAlarmStats(alarmStat, resultQuery)).toBe(true);
  });

  it('should return false when condition is not met', () => {
    const alarmStat: InsightModel = {
      ...baseInsight,
      conditionsCheck: [
        {
          nodeLabel: 'TestNode',
          statistic: 'count',
          operator: '<',
          value: 5,
        },
      ],
    };

    const resultQuery: GraphQueryResultMetaFromBackend = {
      nodes: [
        {
          _id: 'node1',
          label: 'TestNode',
          attributes: {
            name: 'Test Node 1',
            value: 123,
          },
        },
        {
          _id: 'node2',
          label: 'TestNode',
          attributes: {
            name: 'Test Node 2',
            value: 456,
          },
        },
      ],
      edges: [
        {
          _id: 'edge1',
          label: 'CONNECTS',
          from: 'node1',
          to: 'node2',
          attributes: {
            weight: 1,
            timestamp: '2025-03-06',
          },
        },
      ],
      metaData: {
        topological: {
          density: 0.5,
          self_loops: 0,
        },
        nodes: {
          count: 2,
          labels: ['TestNode'],
          types: {
            TestNode: {
              count: 2,
              avgDegreeIn: 0.5,
              avgDegreeOut: 0.5,
              attributes: {
                name: {
                  attributeType: 'string',
                  statistics: {
                    uniqueItems: 2,
                    values: ['Test Node 1', 'Test Node 2'],
                    mode: 'Test Node 1',
                    count: 2,
                  },
                },
                value: {
                  attributeType: 'number',
                  statistics: {
                    min: 123,
                    max: 456,
                    average: 289.5,
                    count: 2,
                  },
                },
              },
            },
          },
        },
        edges: {
          count: 1,
          labels: ['CONNECTS'],
          types: {
            CONNECTS: {
              count: 1,
              attributes: {
                weight: {
                  attributeType: 'number',
                  statistics: {
                    min: 1,
                    max: 1,
                    average: 1,
                    count: 1,
                  },
                },
                timestamp: {
                  attributeType: 'datetime',
                  statistics: {
                    min: 1743782400,
                    max: 1743782400,
                    range: 0,
                  },
                },
              },
            },
          },
        },
      },
      nodeCounts: {
        TestNode: 2,
        updatedAt: 122331,
      },
    };

    expect(processAlarmStats(alarmStat, resultQuery)).toBe(false);
  });

  it('should return false when nodeLabel is not found', () => {
    const alarmStat: InsightModel = {
      ...baseInsight,
      conditionsCheck: [
        {
          nodeLabel: 'NonExistentNode',
          statistic: 'count',
          operator: '>',
          value: 5,
        },
      ],
    };

    const resultQuery: GraphQueryResultMetaFromBackend = {
      nodes: [
        {
          _id: 'node1',
          label: 'TestNode',
          attributes: {
            name: 'Test Node 1',
            value: 123,
          },
        },
        {
          _id: 'node2',
          label: 'TestNode',
          attributes: {
            name: 'Test Node 2',
            value: 456,
          },
        },
      ],
      edges: [
        {
          _id: 'edge1',
          label: 'CONNECTS',
          from: 'node1',
          to: 'node2',
          attributes: {
            weight: 1,
            timestamp: '2025-03-06',
          },
        },
      ],
      metaData: {
        topological: {
          density: 0.5,
          self_loops: 0,
        },
        nodes: {
          count: 2,
          labels: ['TestNode'],
          types: {
            TestNode: {
              count: 2,
              avgDegreeIn: 0.5,
              avgDegreeOut: 0.5,
              attributes: {
                name: {
                  attributeType: 'string',
                  statistics: {
                    uniqueItems: 2,
                    values: ['Test Node 1', 'Test Node 2'],
                    mode: 'Test Node 1',
                    count: 2,
                  },
                },
                value: {
                  attributeType: 'number',
                  statistics: {
                    min: 123,
                    max: 456,
                    average: 289.5,
                    count: 2,
                  },
                },
              },
            },
          },
        },
        edges: {
          count: 1,
          labels: ['CONNECTS'],
          types: {
            CONNECTS: {
              count: 1,
              attributes: {
                weight: {
                  attributeType: 'number',
                  statistics: {
                    min: 1,
                    max: 1,
                    average: 1,
                    count: 1,
                  },
                },
                timestamp: {
                  attributeType: 'datetime',
                  statistics: {
                    min: 1743782400,
                    max: 1743782400,
                    range: 0,
                  },
                },
              },
            },
          },
        },
      },
      nodeCounts: {
        TestNode: 2,
        updatedAt: 122331,
      },
    };

    expect(processAlarmStats(alarmStat, resultQuery)).toBe(false);
  });

  it('should throw an error for unsupported operator', () => {
    const alarmStat: InsightModel = {
      ...baseInsight,
      conditionsCheck: [
        {
          nodeLabel: 'TestNode',
          statistic: 'count',
          operator: 'unsupported',
          value: 5,
        },
      ],
    };

    const resultQuery: GraphQueryResultMetaFromBackend = {
      nodes: [
        {
          _id: 'node1',
          label: 'TestNode',
          attributes: {
            name: 'Test Node 1',
            value: 123,
          },
        },
        {
          _id: 'node2',
          label: 'TestNode',
          attributes: {
            name: 'Test Node 2',
            value: 456,
          },
        },
      ],
      edges: [
        {
          _id: 'edge1',
          label: 'CONNECTS',
          from: 'node1',
          to: 'node2',
          attributes: {
            weight: 1,
            timestamp: '2025-03-06',
          },
        },
      ],
      metaData: {
        topological: {
          density: 0.5,
          self_loops: 0,
        },
        nodes: {
          count: 2,
          labels: ['TestNode'],
          types: {
            TestNode: {
              count: 2,
              avgDegreeIn: 0.5,
              avgDegreeOut: 0.5,
              attributes: {
                name: {
                  attributeType: 'string',
                  statistics: {
                    uniqueItems: 2,
                    values: ['Test Node 1', 'Test Node 2'],
                    mode: 'Test Node 1',
                    count: 2,
                  },
                },
                value: {
                  attributeType: 'number',
                  statistics: {
                    min: 123,
                    max: 456,
                    average: 289.5,
                    count: 2,
                  },
                },
              },
            },
          },
        },
        edges: {
          count: 1,
          labels: ['CONNECTS'],
          types: {
            CONNECTS: {
              count: 1,
              attributes: {
                weight: {
                  attributeType: 'number',
                  statistics: {
                    min: 1,
                    max: 1,
                    average: 1,
                    count: 1,
                  },
                },
                timestamp: {
                  attributeType: 'datetime',
                  statistics: {
                    min: 1743782400,
                    max: 1743782400,
                    range: 0,
                  },
                },
              },
            },
          },
        },
      },
      nodeCounts: {
        TestNode: 2,
        updatedAt: 122331,
      },
    };

    expect(() => processAlarmStats(alarmStat, resultQuery)).toThrow('Unsupported operator: unsupported');
  });
});
